"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/example-greeting-app
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@loopback/core");
const debugFactory = require("debug");
const debug = debugFactory('greeter-extension');
/**
 * Message caching service
 */
let CachingService = class CachingService {
    constructor(optionsView) {
        this.optionsView = optionsView;
        this.store = new Map();
        // Use a view so that we can listen on `refresh` events, which are emitted
        // when the configuration binding is updated in the context.
        optionsView.on('refresh', async () => {
            debug('Restarting the service as configuration changes...');
            await this.stop();
            await this.start();
        });
    }
    // We intentionally mark all operations `async` to reflect the commonly used
    // caching infrastructure even though our in-memory implementation is based
    // on a `Map` that provides synchronous APIs.
    /**
     * Store a message in the cache
     * @param key - Key for caching
     * @param message - Message
     */
    async set(key, message) {
        this.store.set(key, message);
    }
    /**
     * Load a message from the cache by key
     * @param key - Key for caching
     */
    async get(key) {
        const expired = await this.isExpired(key);
        debug('Getting cache for %s', key, expired);
        return expired ? undefined : this.store.get(key);
    }
    /**
     * Delete a message from the cache by key
     * @param key - Key for caching
     */
    async delete(key) {
        return this.store.delete(key);
    }
    /**
     * Clear the cache
     */
    async clear() {
        this.store.clear();
    }
    /**
     * Check if the cached item is expired by key
     * @param key - Key for caching
     * @param now - The current date
     */
    async isExpired(key, now = new Date()) {
        const ttl = await this.getTTL();
        const msg = this.store.get(key);
        if (!msg)
            return true;
        return now.getTime() - msg.timestamp.getTime() > ttl;
    }
    /**
     * Get the TTL setting
     */
    async getTTL() {
        const options = await this.optionsView.singleValue();
        debug('Caching options: %j', options);
        return (options && options.ttl) || 5000;
    }
    /**
     * Remove expired items from the cache
     */
    async sweep() {
        debug('Sweeping cache...');
        for (const key of this.store.keys()) {
            if (await this.isExpired(key)) {
                debug('Cache for %s is swept.', key);
                await this.delete(key);
            }
        }
    }
    /**
     * This method will be invoked when the application starts
     */
    async start() {
        debug('Starting caching service');
        await this.clear();
        const ttl = await this.getTTL();
        debug('TTL: %d', ttl);
        this.timer = setInterval(async () => {
            await this.sweep();
        }, ttl);
    }
    /**
     * This method will be invoked when the application stops
     */
    async stop() {
        debug('Stopping caching service');
        /* istanbul ignore if */
        if (this.timer) {
            clearInterval(this.timer);
        }
        await this.clear();
    }
};
CachingService = __decorate([
    core_1.bind({ scope: core_1.BindingScope.SINGLETON }),
    __param(0, core_1.config.view()),
    __metadata("design:paramtypes", [core_1.ContextView])
], CachingService);
exports.CachingService = CachingService;
//# sourceMappingURL=caching-service.js.map